@@ -4,8 +4,7 @@
 #                                                                                 
 #    This program is free software; you can redistribute it and/or modify         
 #    it under the terms of the GNU General Public License as published by         
-#    the Free Software Foundation; either version 2 of the License, or            
-#    (at your option) any later version.                                          
+#    the Free Software Foundation, version 2.                                      
 #                                                                                 
 #    This program is distributed in the hope that it will be useful,              
 #    but WITHOUT ANY WARRANTY; without even the implied warranty of               
@@ -24,7 +23,7 @@ __version__   = '$ Revision 0.0 $'[11:-2]
 from zope.interface import Interface, implements 
 from zope.component import adapts,  getUtility
 from zope.formlib import form
-from zope.schema import TextLine
+from zope.schema import TextLine, Bool
 from Products.CMFDefault.formlib.schema import SchemaAdapterBase
 from Products.CMFPlone.interfaces import IPloneSiteRoot
 from collective.zipfiletransport import ZipFileTransportMessageFactory as _
@@ -36,26 +35,29 @@ class IZipFileTransportPrefsForm(Interface):
     """ The view for zipfile transport prefs form. """
 
     image_type = TextLine(title=_(u'Image Type'),
-                          description=_(u'Set the image type here. Images imported via ZipFile Transport '
+                          description=_(u'Set the image type here. Images imported via ZipFileTransport '
                                         'will be instantiated as an object of this type.'),
                           required=True)
                           
     file_type = TextLine(title=_(u'File Type'),
-                          description=_(u'Set your file type here. Files imported via ZipFile Transport '
+                          description=_(u'Set your file type here. Files imported via ZipFileTransport '
                                         'will be instantiated as an object of this type'),
                           required=True)
                           
     doc_type = TextLine(title=_(u'Document Type'),
-                          description=_(u'Set your document type here. Documents imported via ZipFile Transport '
+                          description=_(u'Set your document type here. Documents imported via ZipFileTransport '
                                         'will be instantiated as an object of this type.'),
                           required=True)
                           
     folder_type = TextLine(title=_(u'Folder Type'),
-                          description=_(u'Set your folderish type here. Folders imported via ZipFile Transport '
+                          description=_(u'Set your folderish type here. Folders imported via ZipFileTransport '
                                         'will be instantiated as an object of this type'),
                           required=True)
                           
-
+    name_by_title = Bool(title=_('Name Objects By Title'),
+                         description=_(u'Use an object''s title as filename in ZIP export instead of ID.'),
+                         required=False)
+                                 
 
 class ZipFileTransportControlPanelAdapter(SchemaAdapterBase):
     """ Control Panel adapter """
@@ -92,12 +94,18 @@ class ZipFileTransportControlPanelAdapter(SchemaAdapterBase):
 
     def set_folder_type(self, folder_type):
         self.zf_props.folder_type = folder_type
+
+    def get_name_by_title(self):
+        return self.zf_props.name_by_title
     
+    def set_name_by_title(self, value):
+        self.zf_props.name_by_title = value
+
     image_type = property(get_image_type, set_image_type)
     file_type = property(get_file_type, set_file_type)
     doc_type = property(get_doc_type, set_doc_type)
     folder_type = property(get_folder_type, set_folder_type)
-
+    name_by_title = property(get_name_by_title, set_name_by_title)
     
 class ZipFileTransportPrefsForm(ControlPanelForm):
     """ The view class for the zipfile transport preferences form. """
@@ -105,9 +113,9 @@ class ZipFileTransportPrefsForm(ControlPanelForm):
     implements(IZipFileTransportPrefsForm)
     form_fields = form.FormFields(IZipFileTransportPrefsForm)
 
-    label = _(u'ZipFile Transport Settings Form')
+    label = _(u'ZipFileTransport Settings Form')
     description = _(u'Configure object type settings for imported objects.  These values should not be adjusted unless your site has overwritten '
                       'the default object types.')
-    form_name = _(u'ZipFile Transport Settings')
+    form_name = _(u'ZipFileTransport Settings')
 
 
